@@ -28,7 +28,7 @@
 pub mod component_config;
 mod legacy;
 
-use gpcas_base::file::{DeserializeFunction, GpcasFileStruct};
+use gpcas_base::file::{deserialize_upgrade_from, DeserializeFunction, GpcasFileStruct};
 use serde::{Deserialize, Serialize};
 
 /// The configuration of an abstract CPU model.
@@ -40,12 +40,16 @@ pub struct CPUModel {
     pub execution_latencies: InstructionLatencies,
     /// Configuration for the fetch stage of the front end.
     pub fetch_config: component_config::FetchConfig,
-    /// The amount of parallel decoders.
-    pub decoder_count: usize,
     /// Configuration for the memory controller.
     pub memory_controller_config: component_config::MemoryControllerConfig,
     /// Caches of the memory hierarchy.
     pub caches: Vec<component_config::CacheConfig>,
+    /// The amount of parallel decoders.
+    pub decoder_count: usize,
+    /// Configuration of the register file.
+    pub reg_file_config: component_config::RegisterFileConfig,
+    /// Configuration of schedulers.
+    pub schedulers: Vec<component_config::SchedulerConfig>,
     /// Configuration for each pipeline from front end to the last stage.
     pub pipelines: Vec<component_config::PipelineConfig>,
 }
@@ -84,9 +88,11 @@ impl CPUModel {
             max_vector_size: 128,
             execution_latencies: InstructionLatencies::default(),
             fetch_config: component_config::FetchConfig::default(),
-            decoder_count: 1,
             memory_controller_config: component_config::MemoryControllerConfig::default(),
             caches: Vec::new(),
+            decoder_count: 1,
+            reg_file_config: Default::default(),
+            schedulers: Vec::new(),
             pipelines: Vec::new(),
         }
     }
@@ -125,8 +131,9 @@ impl InstructionLatencies {
 
 impl GpcasFileStruct for CPUModel {
     const FILE_IDENTIFIER: &'static str = "gpcas::cpu_model";
-    const CURRENT_FILE_VERSION: usize = 0;
-    const COMPATIBLE_VERSIONS: &'static [(usize, DeserializeFunction<CPUModel>)] = &[];
+    const CURRENT_FILE_VERSION: usize = 1;
+    const COMPATIBLE_VERSIONS: &'static [(usize, DeserializeFunction<CPUModel>)] =
+        &[(0, deserialize_upgrade_from::<legacy::CPUModelV0, CPUModel>)];
 }
 
 /// The default latency in clock cycles for all instruction types.
@@ -134,8 +141,6 @@ impl GpcasFileStruct for CPUModel {
 /// Types not mentioned in here either share their value with another one, or are hardcoded to one
 /// clock cycle.
 pub mod default_latencies {
-    /// Simple addition.
-    pub const INT_ADD: u16 = 1;
     /// Typically takes multiple, but few clock cycles.
     pub const INT_MUL: u16 = 3;
     /// Takes many clock cycles, usually dependent on the operand size.
@@ -152,6 +157,4 @@ pub mod default_latencies {
     pub const FLOAT_DIV: u16 = 22;
     /// Might take multiple ports if a design has separate add- and mul-pipes.
     pub const FLOAT_MUL_ADD: u16 = 5;
-    /// Completes fast, but needs a branch unit.
-    pub const BRANCH: u16 = 1;
 }
